@@ -1,5 +1,6 @@
 import boto3
 from aws.region import get_ec2, get_ssm, get_route53
+from aws.util import format_filters
 import time
 
 def list_regions():
@@ -214,25 +215,18 @@ def get_vpc(default: bool = False, name: str = None, vpcid: str = None):
     :param vpcid: vpc_id to retrieve.
     :return: `Returns the dictionary of VPC info. <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_vpcs>`_
     """
+    assert default or name is not None or vpcid is not None, \
+        'default, name or vpcid must be provided when calling get_vpc()'
+
     vpcs = list_vpcs()
 
-    if default:
-        for vpc in vpcs:
-            if vpc['IsDefault']:
-                return vpc
+    for vpc in vpcs:
+        found_default = default and vpc['IsDefault']
+        found_name = name is not None and match_tags({'Name': name}, vpc)
+        found_id = vpcid is not None and vpc.get('VpcId') == vpcid
 
-    elif name is not None:
-        for vpc in vpcs:
-            if match_tags({'Name': name}, vpc):
-                return vpc
-
-    elif vpcid is not None:
-        for vpc in vpcs:
-            if vpc.get('VpcId') == vpcid:
-                return vpc
-
-    else:
-        raise Exception('default, name or vpcid must be provided when calling get_vpc()')
+        if found_default or found_name or found_id:
+            return vpc
     return None
 
 
@@ -301,6 +295,44 @@ def get_subnet_id(subnet: dict):
     return subnet.get('SubnetId')
 
 
+
+def get_subnets(subnet_ids):
+    """
+    Get detail of subnets by Ids
+    :param subnet_ids: array of subnet ids
+    :return: `List of subnet dictionaries <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_subnets>`_
+    """
+    return get_ec2().describe_subnets(SubnetIds=subnet_ids).get('Subnets')
+
+
+def is_public_subnet(id:str):
+    """
+    Determine if the specified subnet id is the public one or not.
+    The subnet is public if internet gateway is attached to a route on a routetable of a subnet.
+    :param id: ID of subnet
+    :return: True if the subnet is public. Otherwise, False
+    """
+    route_tables = get_ec2().describe_route_tables(
+        Filters=[
+            {
+                'Name': 'association.subnet-id',
+                'Values': [
+                    id
+                ]
+            }
+        ]
+    ).get('RouteTables', [])
+
+    for route_table in route_tables:
+        routes = route_table['Routes']
+        
+        for route in routes:
+            gid = route.get('GatewayId', '')
+            if gid.startswith('igw-'):
+                return True
+
+    return False
+
 ###########################################################################
 # Internet Gateways & NATs
 ###########################################################################
@@ -460,6 +492,27 @@ def list_eips():
     return get_ec2().describe_addresses()['Addresses']
 
 
+def release_eip(allocation_id: str):
+    """
+    Releases an Elastic IP
+
+    :param id: The AllocationId of the Elastic IP to delete
+    :return: None
+    """
+    get_ec2().release_address(AllocationId=allocation_id)
+
+
+def release_unused_eips():
+    """
+    Releases all unused Elastic IPs (those with no AssociationId).
+
+    :return: None
+    """
+    for eip in list_eips():
+        if 'AssociationId' not in eip:
+            release_eip(eip['AllocationId'])
+
+
 ###########################################################################
 # Route Tables
 ###########################################################################
@@ -712,32 +765,39 @@ def get_route_table(name: str = None, rt_id: str = None, subnet_id: str = None):
     """
     rts = list_route_tables()
 
-    if name is not None:
-        for rt in rts:
-            if match_tags({'Name': name}, rt):
-                return rt
+    assert name is not None or rt_id is not None or subnet_id is not None, \
+        'name, rt_id or subnet_id must be specified when calling get_route_table()'
 
-    elif rt_id is not None:
-        for rt in rts:
-            if rt_id == rt.get('RouteTableId'):
-                return rt
+    for rt in rts:
 
-    elif subnet_id is not None:
-        for rt in rts:
-            associations = rt.get('Associations')
-            if associations is None:
-                return None
+        name_found = name is not None and match_tags({'Name': name}, rt)
+        id_found = rt_id is not None and rt_id == rt.get('RouteTableId')
+        subnet_id_found = subnet_id is not None and check_subnet_id_in_route_table(subnet_id, rt)
 
-            for association in associations:
-                if subnet_id == association.get('SubnetId'):
-                    return rt
-
-    else:
-        raise Exception('name, rt_id or subnet_id must be specified when calling get_route_table()')
+        if name_found or id_found or subnet_id_found:
+            return rt
 
     return None
 
 
+def check_subnet_id_in_route_table(subnet_id, rt):
+    """
+    Checks if a subnet is in a route table's associations
+
+    :param subnet_id: The id to find in the route_table
+    :param rt: the route table dictionary object
+
+    :returns: True if subnet is in route table's associations list. False otherwise.
+    """
+    associations = rt.get('Associations')
+    if associations is None:
+        return False
+
+    for association in associations:
+        if subnet_id == association.get('SubnetId'):
+            return True
+    return False
+
 def get_route_table_id(rt: dict):
     """
     Extract RouteTableId from the route table dictionary definition.
@@ -746,6 +806,44 @@ def get_route_table_id(rt: dict):
     """
     return rt.get('RouteTableId')
 
+
+def get_main_route_table():
+    """
+    Get the main route table associated with the current VPC.
+
+    :return: dict representing the route table
+
+    Example output::
+
+        {
+            Associations: [
+                {
+                    Main: True
+                    RouteTableAssociationId: "rtbassoc-098aab5b617465c86"
+                    RouteTableId: "rtb-0142fd63a657ef8cf"
+                }
+            ]
+            PropagatingVgws: []
+            RouteTableId: "rtb-0142fd63a657ef8cf"
+            Routes: [
+                {
+                    DestinationCidrBlock: "10.222.0.0/16"
+                    GatewayId: "local"
+                    Origin: "CreateRouteTable"
+                    State: "active"
+                }
+            ]
+            Tags: []
+            VpcId: "vpc-08bc3183e3bc9e88e"
+            OwnerId: "862586795542"
+        }
+    """
+    for rt in list_route_tables():
+        for a in rt.get('Associations', []):
+            if a['Main']:
+                return rt
+
+
 ###########################################################################
 # Security Groups
 ###########################################################################
@@ -1402,28 +1500,22 @@ def match_tags(tags: dict, aws_obj: dict):
 
     return True
 
-def set_tag(key: str, value: str, name: str = None, instance_id: str = None):
+
+def set_tag(key: str, value: str, resource_id: str):
     """
-    Sets a tag for an EC2 instance to a given value.
+    Sets a tag for an EC2 resource to a given value.
 
     :param key: The tag key to set
     :param value: The new/updated tag's value
-    :param name: The instance's name. Required if parameter instance_id is not supplied.
-    :param instance_id: The instance's id. Required if parameter name is not supplied.
-    :return: True if the instance is successfully updated with a tag key = value.
-             None if both name and id are missing or if the corresponding instance doesn't exist.
+    :param resource_id: The resource's id
+    :return: True if the resource is successfully updated with a tag key = value.
+             None if the corresponding resource doesn't exist.
     """
-    if name is None and instance_id is None:
-        print('Must supply an instance name or id to tag')
-        return None
-    instance = get_instance(name = name, instance_id = instance_id)
-    if instance == None:
-        return None
     tag = {
       'Key': key,
       'Value': value
     }
-    get_ec2().create_tags(Resources = [get_instance_id(instance)], Tags = [tag])
+    get_ec2().create_tags(Resources = [resource_id], Tags = [tag])
     return True
     
 
@@ -1518,4 +1610,53 @@ def get_zone_vpc_ids(zone_id: str):
     return [
       vpc.get('VPCId')
       for vpc in vpcs
-    ]
\ No newline at end of file
+    ]
+
+
+###########################################################################
+# Network ACL
+###########################################################################
+
+def get_default_network_acl():
+    """
+    Return the default network ACL for the current VPC
+
+    :return: A dict representing the network ACL or None if not found
+
+    Example output::
+
+        {
+            "Associations": [
+                {
+                    "NetworkAclAssociationId": "aclassoc-ddf3c181",
+                    "NetworkAclId": "acl-0f0e1fca3cf9786bc",
+                    "SubnetId": "subnet-0ddd980c49b0b7381"
+                },
+                ...
+            ],
+            "Entries": [
+                {
+                    "CidrBlock": "0.0.0.0/0",
+                    "Egress": true,
+                    "Protocol": "-1",
+                    "RuleAction": "allow",
+                    "RuleNumber": 100
+                },
+                ...
+            ],
+            "IsDefault": true,
+            "NetworkAclId": "acl-0f0e1fca3cf9786bc",
+            "Tags": [],
+            "VpcId": "vpc-0dbb013f82b6694e9",
+            "OwnerId": "683863474030"
+        }
+
+    """
+    filters = format_filters({
+        'vpc-id': get_current_vpcid(),
+        'default': 'true'
+    })
+
+    acls = get_ec2().describe_network_acls(Filters=filters).get('NetworkAcls')
+
+    return acls[0] if acls else None
\ No newline at end of file
