@@ -153,8 +153,8 @@ def list_classic_loadbalancer():
         ]
 
     """
-    elbList = get_elb().describe_load_balancers().get('LoadBalancerDescriptions')
-    return elbList
+    elbs = get_elb().describe_load_balancers().get('LoadBalancerDescriptions')
+    return elbs
 
 
 def get_classic_loadbalancer(name: str = None):
@@ -232,9 +232,9 @@ def get_classic_loadbalancer(name: str = None):
         ]
 
     """
-    elbList = list_classic_loadbalancer()
+    elbs = list_classic_loadbalancer()
     if name is not None:
-        for elb in elbList:
+        for elb in elbs:
             if elb.get('LoadBalancerName') == name:
                 return elb
     else:
@@ -386,8 +386,8 @@ def list_application_loadbalancer():
         ]
 
     """
-    elbList = get_elbv2().describe_load_balancers().get('LoadBalancers')
-    return elbList
+    elbs = get_elbv2().describe_load_balancers().get('LoadBalancers')
+    return elbs
 
 
 def get_application_loadbalancer(arn: str = None, name: str = None):
@@ -448,13 +448,13 @@ def get_application_loadbalancer(arn: str = None, name: str = None):
 
     """
 
-    elbList = list_application_loadbalancer()
+    elbs = list_application_loadbalancer()
     if name is not None:
-        for elb in elbList:
+        for elb in elbs:
             if elb.get('LoadBalancerName') == name:
                 return elb
     elif arn is not None:
-        for elb in elbList:
+        for elb in elbs:
             if elb.get('LoadBalancerArn') == arn:
                 return elb
     else:
@@ -538,8 +538,8 @@ def list_target_groups():
         ]
 
     """
-    targetGroupList = get_elbv2().describe_target_groups().get('TargetGroups')
-    return targetGroupList
+    target_groups = get_elbv2().describe_target_groups().get('TargetGroups')
+    return target_groups
 
 
 def get_target_group(name: str = None, arn: str = None):
@@ -549,18 +549,19 @@ def get_target_group(name: str = None, arn: str = None):
     :param arn: The ARN of the Target Group. Required if parameter name isn't supplied.
     :return: `TargetGroup Details <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_target_groups>`_
     """
-    tgList = list_target_groups()
+    tgs = list_target_groups()
     if name is not None:
-        for tg in tgList:
+        for tg in tgs:
             if tg.get('TargetGroupName') == name:
                 return tg
     elif arn is not None:
-        for tg in tgList:
+        for tg in tgs:
             if tg.get('TargetGroupArn') == arn:
                 return tg
     else:
         raise Exception('Either name or arn of the target group must be provided for the search')
 
+
 def add_target(port: int = None, target_name: str = None, target_arn: str = None, group_name: str = None, group_arn: str = None):
     """
     Adds a target to a target group.
@@ -582,25 +583,25 @@ def add_target(port: int = None, target_name: str = None, target_arn: str = None
     # make sure the instance exists
     if target_name is not None:
         target = get_instance(target_name)
-        if target is None:
-            print(f"instance {target_name} not found")
-            return False
+        target_instance_name = target_name
     else: # arn
         target = get_instance(instance_id = target_arn)
-        if target is None:
-            print(f"instance {target_arn} not found")
-            return False
+        target_instance_name = target_arn
+    if target is None:
+        print(f"instance {target_instance_name} not found")
+        return False
+
     # make sure the target group exists
     if group_name is not None:
         group = get_target_group(group_name)
-        if group is None:
-            print(f"group {group_name} not found")
-            return False
+        group_instance_name = group_name
     else: # arn
         group = get_target_group(arn = group_arn)
-        if group is None:
-            print(f"group {group_arn} not found")
-            return False
+        group_instance_name = group_arn
+    if group is None:
+        print(f"group {group_instance_name} not found")
+        return False
+
     # add the retrieved target to the target group
     if port is None:
         port = group['Port']
@@ -612,6 +613,7 @@ def add_target(port: int = None, target_name: str = None, target_arn: str = None
 
     return True
 
+
 def get_targets_in_group(group_name: str = None, group_arn: str = None):
     """
     Gets a list of targets in a group.
@@ -655,9 +657,9 @@ def get_elb_tags(arn: str) -> list:
 
     """
 
-    tagList = get_elbv2().describe_tags(ResourceArns=[arn]).get('TagDescriptions')[0].get('Tags')
+    tags = get_elbv2().describe_tags(ResourceArns=[arn]).get('TagDescriptions')[0].get('Tags')
     res = {}
-    for x in tagList:
+    for x in tags:
         key = x.get('Key')
         val = x.get('Value')
         res[key] = val
