@@ -1,8 +1,5 @@
 from unittest import TestCase, mock, main
 from aws.iam import *
-from po.pretty_object import PrettyObject
-
-po = PrettyObject()
 
 
 class TestIam(TestCase):
